"use strict";
var subscriber_set_1 = require("./subscriber-set");
var transitioner_1 = require("./transitioner");
var transition_set_1 = require("./transition-set");
var StateMachine = (function () {
    function StateMachine(initialState) {
        this.currentState = initialState;
        this.subscribers = new subscriber_set_1.default();
        this.transitions = new transition_set_1.default();
        this.transitioner = new transitioner_1.default(this.transitions);
    }
    StateMachine.create = function (initialState) {
        return new StateMachine(initialState);
    };
    StateMachine.prototype.getState = function () {
        return this.currentState;
    };
    StateMachine.prototype.addEvent = function (name, transitions) {
        this.transitions.addEvent({ name: name, transitions: transitions });
    };
    StateMachine.prototype.removeEvent = function (name) {
        this.transitions.removeEvent(name);
    };
    StateMachine.prototype.hasEvent = function (name) {
        return this.transitions.hasEvent(name);
    };
    StateMachine.prototype.addTransition = function (transition) {
        this.transitions.addTransition(transition);
    };
    StateMachine.prototype.removeTransition = function (transition) {
        this.transitions.removeTransition(transition);
    };
    StateMachine.prototype.hasTransition = function (transition) {
        return this.transitions.hasTransition(transition);
    };
    StateMachine.prototype.canTransition = function (state) {
        return this.transitioner.canTransition(state, this.currentState);
    };
    StateMachine.prototype.transition = function (state) {
        var _this = this;
        this.transitioner.transition(state, this.currentState, function (transition) {
            _this.currentState = transition.to;
            _this.subscribers.notifySubscribers({
                from: transition.from,
                to: transition.to,
            });
        });
    };
    StateMachine.prototype.transitionByEvent = function (event) {
        var _this = this;
        this.transitioner.transitionByEvent(event, this.currentState, function (transition) {
            _this.currentState = transition.to;
            _this.subscribers.notifySubscribers({
                event: event,
                from: transition.from,
                to: transition.to,
            });
        });
    };
    StateMachine.prototype.subscribe = function (subscriber) {
        this.subscribers.addSubscriber(subscriber);
    };
    StateMachine.prototype.unsubscribe = function (subscriber) {
        this.subscribers.removeSubscriber(subscriber);
    };
    return StateMachine;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = StateMachine;
//# sourceMappingURL=state-machine.js.map